'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var html = require('../html');
var fuco = require('../fuco');

function rehydrateScript() {
    let scripts = "";
    scripts += "var e=function(a,c){for(var i=0;i<a.length;i++)c(a[i])},";
    scripts += "d=function(a){a&&a.parentNode.removeChild(a)},";
    scripts +=
        "r=function(n){return function(){" +
            "var s=document.querySelectorAll(n);" +
            "e(s,function(a){e(a.querySelectorAll('shadowroot'),function(b){" +
            "e(b,function(c){e(c.querySelectorAll('slot'),function(d){s.appendChild(d)})});d(b)})});" +
            "d(__ssr__)" +
            "};};";
    for (const elementName in fuco.__FucoRegistry__) {
        scripts += `customElements.whenDefined("${elementName}").then(r("${elementName}"));`;
    }
    return `<script>var __ssr__=document.currentScript;(function(){${scripts}}())</script>`;
}

/* eslint-disable @typescript-eslint/no-empty-function */
class Component {
    constructor(fc, props, args) {
        this._hooks = fuco.defaultHooks();
        this.props = createCmpProps(this, props, args);
        fuco.__setCurrent__(this);
        this.result = fc();
    }
    _attr(name, converter) {
        const value = this.props[name] != null ? String(this.props[name]) : null;
        return converter ? converter(value) : value;
    }
    _observeAttr() { }
    _dispatch() { }
    _adoptStyle() { }
}
function createCmpProps(component, props, args) {
    const cmpProps = {};
    if (props) {
        const resolveArgs = (_props) => {
            for (const key in _props) {
                let value = _props[key];
                if (typeof value === "number" && args) {
                    value = args[value];
                }
                if (key[0] === "@" || key === ":key" || key === ":ref") {
                    continue;
                }
                else if (key === "...") {
                    resolveArgs(value);
                }
                else if (key[0] === "?") {
                    cmpProps[key.slice(1)] = value ? "" : null;
                }
                else if (key[0] === ".") {
                    component[key.slice(1)] = value;
                }
                else {
                    cmpProps[key] = value;
                }
            }
        };
        resolveArgs(props);
    }
    return cmpProps;
}

function compose(vnode, args) {
    let fc;
    if (!(fc = fuco.__FucoRegistry__[vnode.tag])) {
        return vnode;
    }
    const { tag, props, children } = vnode;
    const c = new Component(fc, props, args);
    props && delete props[".innerHTML"];
    const shadowroot = {
        tag: "shadowroot",
        children: replaceSlot(1, [, c.result], children)
    };
    return { tag, props, children: [shadowroot] };
}
function replaceSlot(vdom, args, slottedNodes, hasNamedSlot = false) {
    if (Array.isArray(vdom)) {
        return vdom.map(v => replaceSlot(v, args, slottedNodes, hasNamedSlot));
    }
    else if (typeof vdom === "number") {
        return replaceSlot(args[vdom], args, slottedNodes, hasNamedSlot);
    }
    else if (html.isTemplate(vdom)) {
        const [v, a] = html.items(vdom);
        return replaceSlot(v, a, slottedNodes, hasNamedSlot);
    }
    else if (html.isVNode(vdom)) {
        if (vdom.tag === "slot") {
            const [slots, namedSlots] = getSlotChildren(slottedNodes);
            if (vdom.props && vdom.props.name) {
                vdom.children = namedSlots[vdom.props.name];
            }
            else {
                vdom.children = slots;
            }
        }
        else {
            vdom.children = vdom.children.map(v => replaceSlot(v, args, slottedNodes, hasNamedSlot));
        }
        return vdom;
    }
    else {
        return vdom;
    }
}
function getSlotChildren(slotted) {
    const slots = [];
    const namedSlots = {};
    for (let i = 0; i < slotted.length; i++) {
        const s = slotted[i];
        let slotName;
        if (html.isVNode(s) && s.props && (slotName = s.props.slot)) {
            if (typeof namedSlots[slotName] === "undefined") {
                namedSlots[slotName] = [];
            }
            namedSlots[slotName].push(s);
        }
        else {
            slots.push(s);
        }
    }
    return [slots, namedSlots];
}

const voidTagNameRegexp = /^(area|base|br|col|embed|hr|img|input|link|meta|param|source|track|wbr)$/i;
function stringify(vdom, args, selectValue) {
    if (Array.isArray(vdom)) {
        return vdom.reduce((acc, v) => acc + stringify(v, args, selectValue), "");
    }
    else if (vdom == null) {
        return "";
    }
    else if (typeof vdom === "number" && args) {
        return stringify(args[vdom], undefined, selectValue);
    }
    else if (html.isTemplate(vdom)) {
        const i = html.items(vdom);
        return stringify(i[0], i[1], selectValue);
    }
    else if (!html.isVNode(vdom)) {
        return String(vdom);
    }
    else {
        let s = "";
        const { tag, props, children } = compose(vdom, args);
        if (props) {
            let attrs = "";
            let html, r;
            const propsToString = (props, args) => {
                for (const name in props) {
                    if (name === ":key" || name === ":ref")
                        continue;
                    let v = props[name];
                    if (typeof v === "number" && args)
                        v = args[v];
                    if (name === "...") {
                        propsToString(v);
                    }
                    else if ((r = name.match(/^(@|\.|\?)([a-zA-Z1-9-]+)/))) {
                        if (r[1] === "?" && v)
                            attrs += ` ${r[2]}`;
                        else if (r[1] === "." && r[2] === "innerHTML")
                            html = v;
                    }
                    else if (v != null) {
                        attrs += ` ${name}="${v}"`;
                    }
                    if (tag === "option" && name === "value" && v === selectValue) {
                        attrs += " selected";
                        selectValue = undefined;
                    }
                    if (tag === "select" && name === ".value" && v != null) {
                        selectValue = v;
                    }
                }
            };
            propsToString(props, args);
            s = `<${tag}${attrs}>`;
            if (html) {
                return `${s}${html}</${tag}>`;
            }
        }
        else {
            s = `<${tag}>`;
        }
        if (!voidTagNameRegexp.test(tag)) {
            s += `${stringify(children, args, selectValue)}</${tag}>`;
        }
        return s;
    }
}

function renderToString(value, options) {
    var _a;
    let html = stringify(1, [0, value]);
    if ((_a = options) === null || _a === void 0 ? void 0 : _a.hydrate) {
        html += rehydrateScript();
    }
    return html;
}

Object.defineProperty(exports, 'html', {
  enumerable: true,
  get: function () {
    return html.html;
  }
});
exports.renderToString = renderToString;
